"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RateLimiterGuard = void 0;
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const default_error_body_creator_1 = require("./default-error-body-creator");
const get_limit_1 = require("./get-limit");
const tokens_1 = require("./tokens");
const too_many_requests_exception_1 = require("./too-many-requests.exception");
let RateLimiterGuard = class RateLimiterGuard {
    constructor(defaultParams, reflector) {
        this.defaultParams = defaultParams;
        this.reflector = reflector;
        this.db = defaultParams.db;
    }
    async canActivate(context) {
        let paramsList = this.reflector.getAllAndOverride(tokens_1.DECORATOR_PARAMS_TOKEN, [context.getHandler(), context.getClass()]);
        if (isTurnedOff(paramsList)) {
            return true;
        }
        if (!paramsList) {
            paramsList = [{}];
        }
        const response = context.switchToHttp().getResponse();
        // in case of `fastify` get native response via `res` property
        // https://www.fastify.io/docs/latest/Reply/#introduction
        // in case of `express` response inherit native response
        const nativeResponse = response.raw || response;
        for (const param of paramsList) {
            await this.checkSingleParam(param, context, nativeResponse);
        }
        return true;
    }
    async checkSingleParam(param, context, response) {
        const getId = param.getId || this.defaultParams.getId;
        if (!getId) {
            return;
        }
        let id;
        try {
            id = await getId(context);
        }
        catch (error) {
            throw new common_1.InternalServerErrorException('Can not get id for rate limiter', String(error));
        }
        const limiterParams = {
            id,
            db: this.db,
            max: param.max || this.defaultParams.max,
            duration: param.duration || this.defaultParams.duration,
        };
        let limit;
        try {
            limit = await (0, get_limit_1.getLimit)(limiterParams);
        }
        catch (error) {
            // Redis error while creating limiter
            throw new common_1.InternalServerErrorException('Can not create rate limiter', String(error));
        }
        setLimitHeaders(response, limit);
        if (limit.remaining < 1) {
            this.onLimitExceed(limit, response, param);
        }
    }
    onLimitExceed(limit, response, param) {
        const after = (limit.reset - Date.now() / 1000) | 0;
        response.setHeader('Retry-After', after);
        const createErrorBody = param.createErrorBody ||
            this.defaultParams.createErrorBody ||
            default_error_body_creator_1.defaultErrorBodyCreator;
        throw new too_many_requests_exception_1.TooManyRequestsException(createErrorBody(limit));
    }
};
RateLimiterGuard = __decorate([
    __param(0, (0, common_1.Inject)(tokens_1.MODULE_PARAMS_TOKEN)),
    __metadata("design:paramtypes", [Object, core_1.Reflector])
], RateLimiterGuard);
exports.RateLimiterGuard = RateLimiterGuard;
function isTurnedOff(params) {
    return !!params && params[0] === false;
}
function setLimitHeaders(response, limit) {
    response.setHeader('X-RateLimit-Limit', limit.total);
    response.setHeader('X-RateLimit-Remaining', limit.remaining - 1);
    response.setHeader('X-RateLimit-Reset', limit.reset);
}
//# sourceMappingURL=guard.js.map